{
    "$graph": [
        {
            "class": "Workflow",
            "doc": "ATAC-seq 03 mapping - reads: PE",
            "requirements": [
                {
                    "class": "ScatterFeatureRequirement"
                },
                {
                    "class": "SubworkflowFeatureRequirement"
                },
                {
                    "class": "StepInputExpressionRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "doc": "Bowtie first index files for reference genome (e.g. *1.ebwt). The rest of the files should be in the same folder.",
                    "type": "File",
                    "id": "#main/genome_ref_first_index_file"
                },
                {
                    "doc": "Genome sizes tab-delimited file (used in samtools)",
                    "type": "File",
                    "id": "#main/genome_sizes_file"
                },
                {
                    "doc": "Input fastq files for paired_read1",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "id": "#main/input_fastq_read1_files"
                },
                {
                    "doc": "Input fastq files for paired_read2",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "id": "#main/input_fastq_read2_files"
                },
                {
                    "default": 1,
                    "type": "int",
                    "id": "#main/nthreads"
                },
                {
                    "default": "/usr/picard/picard.jar",
                    "doc": "Picard Java jar file",
                    "type": "string",
                    "id": "#main/picard_jar_path"
                },
                {
                    "doc": "JVM arguments should be a quoted, space separated list (e.g. \"-Xms128m -Xmx512m\")",
                    "type": [
                        "null",
                        "string"
                    ],
                    "id": "#main/picard_java_opts"
                }
            ],
            "steps": [
                {
                    "run": "#bowtie-pe.cwl",
                    "scatterMethod": "dotproduct",
                    "scatter": [
                        "#main/bowtie-pe/input_fastq_read1_file",
                        "#main/bowtie-pe/input_fastq_read2_file",
                        "#main/bowtie-pe/output_filename"
                    ],
                    "in": [
                        {
                            "valueFrom": "${return 2000}",
                            "id": "#main/bowtie-pe/X"
                        },
                        {
                            "source": "#main/genome_ref_first_index_file",
                            "id": "#main/bowtie-pe/genome_ref_first_index_file"
                        },
                        {
                            "source": "#main/input_fastq_read1_files",
                            "id": "#main/bowtie-pe/input_fastq_read1_file"
                        },
                        {
                            "source": "#main/input_fastq_read2_files",
                            "id": "#main/bowtie-pe/input_fastq_read2_file"
                        },
                        {
                            "source": "#main/nthreads",
                            "id": "#main/bowtie-pe/nthreads"
                        },
                        {
                            "source": "#main/extract_basename_2/output_path",
                            "id": "#main/bowtie-pe/output_filename"
                        },
                        {
                            "valueFrom": "${return 2}",
                            "id": "#main/bowtie-pe/v"
                        }
                    ],
                    "out": [
                        "#main/bowtie-pe/output_aligned_file",
                        "#main/bowtie-pe/output_bowtie_log"
                    ],
                    "id": "#main/bowtie-pe"
                },
                {
                    "run": "#samtools-idxstats.cwl",
                    "scatter": "#main/dedup_bam_idxstats/bam",
                    "in": [
                        {
                            "source": "#main/index_dedup_bams/indexed_file",
                            "id": "#main/dedup_bam_idxstats/bam"
                        }
                    ],
                    "out": [
                        "#main/dedup_bam_idxstats/idxstats_file"
                    ],
                    "id": "#main/dedup_bam_idxstats"
                },
                {
                    "in": [
                        {
                            "source": "#main/genome_sizes_file",
                            "id": "#main/execute_pcr_bottleneck_coef/genome_sizes"
                        },
                        {
                            "source": "#main/filtered2sorted/sorted_file",
                            "id": "#main/execute_pcr_bottleneck_coef/input_bam_files"
                        },
                        {
                            "source": "#main/extract_basename_2/output_path",
                            "id": "#main/execute_pcr_bottleneck_coef/input_output_filenames"
                        }
                    ],
                    "run": "#pcr-bottleneck-coef.cwl",
                    "out": [
                        "#main/execute_pcr_bottleneck_coef/pbc_file"
                    ],
                    "id": "#main/execute_pcr_bottleneck_coef"
                },
                {
                    "run": "#extract-basename.cwl",
                    "scatter": "#main/extract_basename_1/input_file",
                    "in": [
                        {
                            "source": "#main/input_fastq_read1_files",
                            "id": "#main/extract_basename_1/input_file"
                        }
                    ],
                    "out": [
                        "#main/extract_basename_1/output_basename"
                    ],
                    "id": "#main/extract_basename_1"
                },
                {
                    "run": "#remove-extension.cwl",
                    "scatter": "#main/extract_basename_2/file_path",
                    "in": [
                        {
                            "source": "#main/extract_basename_1/output_basename",
                            "id": "#main/extract_basename_2/file_path"
                        }
                    ],
                    "out": [
                        "#main/extract_basename_2/output_path"
                    ],
                    "id": "#main/extract_basename_2"
                },
                {
                    "run": "#samtools-filter-unmapped.cwl",
                    "scatterMethod": "dotproduct",
                    "scatter": [
                        "#main/filter-unmapped/input_file",
                        "#main/filter-unmapped/output_filename"
                    ],
                    "in": [
                        {
                            "source": "#main/sort_bams/sorted_file",
                            "id": "#main/filter-unmapped/input_file"
                        },
                        {
                            "source": "#main/extract_basename_2/output_path",
                            "id": "#main/filter-unmapped/output_filename"
                        }
                    ],
                    "out": [
                        "#main/filter-unmapped/filtered_file"
                    ],
                    "id": "#main/filter-unmapped"
                },
                {
                    "run": "#samtools-sort.cwl",
                    "scatter": "#main/filtered2sorted/input_file",
                    "in": [
                        {
                            "source": "#main/filter-unmapped/filtered_file",
                            "id": "#main/filtered2sorted/input_file"
                        },
                        {
                            "source": "#main/nthreads",
                            "id": "#main/filtered2sorted/nthreads"
                        }
                    ],
                    "out": [
                        "#main/filtered2sorted/sorted_file"
                    ],
                    "id": "#main/filtered2sorted"
                },
                {
                    "run": "#samtools-index.cwl",
                    "scatter": "#main/index_dedup_bams/input_file",
                    "in": [
                        {
                            "source": "#main/sort_dedup_bams/sorted_file",
                            "id": "#main/index_dedup_bams/input_file"
                        }
                    ],
                    "out": [
                        "#main/index_dedup_bams/indexed_file"
                    ],
                    "id": "#main/index_dedup_bams"
                },
                {
                    "run": "#extract-basename.cwl",
                    "in": [
                        {
                            "source": "#main/remove_duplicates/output_dedup_bam_file",
                            "id": "#main/mapped_file_basename/input_file"
                        }
                    ],
                    "scatter": "#main/mapped_file_basename/input_file",
                    "out": [
                        "#main/mapped_file_basename/output_basename"
                    ],
                    "id": "#main/mapped_file_basename"
                },
                {
                    "run": "#samtools-extract-number-mapped-reads.cwl",
                    "scatter": "#main/mapped_filtered_reads_count/input_bam_file",
                    "in": [
                        {
                            "source": "#main/index_dedup_bams/indexed_file",
                            "id": "#main/mapped_filtered_reads_count/input_bam_file"
                        },
                        {
                            "valueFrom": ".mapped_and_filtered.read_count.txt",
                            "id": "#main/mapped_filtered_reads_count/output_suffix"
                        }
                    ],
                    "out": [
                        "#main/mapped_filtered_reads_count/output_read_count"
                    ],
                    "id": "#main/mapped_filtered_reads_count"
                },
                {
                    "run": "#bowtie-log-read-count.cwl",
                    "scatter": "#main/mapped_reads_count/bowtie_log",
                    "in": [
                        {
                            "source": "#main/bowtie-pe/output_bowtie_log",
                            "id": "#main/mapped_reads_count/bowtie_log"
                        }
                    ],
                    "out": [
                        "#main/mapped_reads_count/output"
                    ],
                    "id": "#main/mapped_reads_count"
                },
                {
                    "run": "#idxstats-percentage-of-reads-in-chrom.cwl",
                    "scatter": "#main/percent_mitochondrial_reads/idxstats",
                    "in": [
                        {
                            "valueFrom": "chrM",
                            "id": "#main/percent_mitochondrial_reads/chrom"
                        },
                        {
                            "source": "#main/dedup_bam_idxstats/idxstats_file",
                            "id": "#main/percent_mitochondrial_reads/idxstats"
                        },
                        {
                            "valueFrom": "${return inputs.idxstats.basename.replace(/^.*[\\\\\\/]/, '').replace(/\\.[^/.]+$/, '').replace(/\\.[^/.]+$/, '.mitochondrial_percentage.txt')}",
                            "id": "#main/percent_mitochondrial_reads/output_filename"
                        }
                    ],
                    "out": [
                        "#main/percent_mitochondrial_reads/percent_map"
                    ],
                    "id": "#main/percent_mitochondrial_reads"
                },
                {
                    "run": "#preseq-percent-uniq-reads.cwl",
                    "scatter": "#main/percent_uniq_reads/preseq_c_curve_outfile",
                    "in": [
                        {
                            "source": "#main/preseq-c-curve/output_file",
                            "id": "#main/percent_uniq_reads/preseq_c_curve_outfile"
                        }
                    ],
                    "out": [
                        "#main/percent_uniq_reads/output"
                    ],
                    "id": "#main/percent_uniq_reads"
                },
                {
                    "run": "#preseq-c_curve.cwl",
                    "scatterMethod": "dotproduct",
                    "scatter": [
                        "#main/preseq-c-curve/input_sorted_file",
                        "#main/preseq-c-curve/output_file_basename"
                    ],
                    "in": [
                        {
                            "source": "#main/filtered2sorted/sorted_file",
                            "id": "#main/preseq-c-curve/input_sorted_file"
                        },
                        {
                            "source": "#main/extract_basename_2/output_path",
                            "id": "#main/preseq-c-curve/output_file_basename"
                        },
                        {
                            "default": true,
                            "id": "#main/preseq-c-curve/pe"
                        }
                    ],
                    "out": [
                        "#main/preseq-c-curve/output_file"
                    ],
                    "id": "#main/preseq-c-curve"
                },
                {
                    "run": "#picard-MarkDuplicates.cwl",
                    "scatterMethod": "dotproduct",
                    "scatter": [
                        "#main/remove_duplicates/input_file",
                        "#main/remove_duplicates/output_filename"
                    ],
                    "in": [
                        {
                            "source": "#main/filtered2sorted/sorted_file",
                            "id": "#main/remove_duplicates/input_file"
                        },
                        {
                            "source": "#main/picard_java_opts",
                            "id": "#main/remove_duplicates/java_opts"
                        },
                        {
                            "source": "#main/extract_basename_2/output_path",
                            "id": "#main/remove_duplicates/output_filename"
                        },
                        {
                            "source": "#main/picard_jar_path",
                            "id": "#main/remove_duplicates/picard_jar_path"
                        }
                    ],
                    "out": [
                        "#main/remove_duplicates/output_metrics_file",
                        "#main/remove_duplicates/output_dedup_bam_file"
                    ],
                    "id": "#main/remove_duplicates"
                },
                {
                    "run": "#samtools2bam.cwl",
                    "scatter": "#main/sam2bam/input_file",
                    "in": [
                        {
                            "source": "#main/bowtie-pe/output_aligned_file",
                            "id": "#main/sam2bam/input_file"
                        },
                        {
                            "source": "#main/nthreads",
                            "id": "#main/sam2bam/nthreads"
                        }
                    ],
                    "out": [
                        "#main/sam2bam/bam_file"
                    ],
                    "id": "#main/sam2bam"
                },
                {
                    "run": "#samtools-sort.cwl",
                    "scatter": "#main/sort_bams/input_file",
                    "in": [
                        {
                            "source": "#main/sam2bam/bam_file",
                            "id": "#main/sort_bams/input_file"
                        },
                        {
                            "source": "#main/nthreads",
                            "id": "#main/sort_bams/nthreads"
                        }
                    ],
                    "out": [
                        "#main/sort_bams/sorted_file"
                    ],
                    "id": "#main/sort_bams"
                },
                {
                    "run": "#samtools-sort.cwl",
                    "scatter": "#main/sort_dedup_bams/input_file",
                    "in": [
                        {
                            "source": "#main/remove_duplicates/output_dedup_bam_file",
                            "id": "#main/sort_dedup_bams/input_file"
                        },
                        {
                            "source": "#main/nthreads",
                            "id": "#main/sort_dedup_bams/nthreads"
                        }
                    ],
                    "out": [
                        "#main/sort_dedup_bams/sorted_file"
                    ],
                    "id": "#main/sort_dedup_bams"
                }
            ],
            "id": "#main",
            "outputs": [
                {
                    "doc": "Bowtie log file.",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/bowtie-pe/output_bowtie_log",
                    "id": "#main/output_bowtie_log"
                },
                {
                    "doc": "BAM files without duplicate reads.",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/index_dedup_bams/indexed_file",
                    "id": "#main/output_data_sorted_dedup_bam_files"
                },
                {
                    "doc": "PCR Bottleneck Coeficient files.",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/execute_pcr_bottleneck_coef/pbc_file",
                    "id": "#main/output_pbc_files"
                },
                {
                    "doc": "Percentage of mitochondrial reads.",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/percent_mitochondrial_reads/percent_map",
                    "id": "#main/output_percent_mitochondrial_reads"
                },
                {
                    "doc": "Percentage of uniq reads from preseq c_curve output",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/percent_uniq_reads/output",
                    "id": "#main/output_percentage_uniq_reads"
                },
                {
                    "doc": "Picard MarkDuplicates metrics files.",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/remove_duplicates/output_metrics_file",
                    "id": "#main/output_picard_mark_duplicates_files"
                },
                {
                    "doc": "Preseq c_curve output files.",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/preseq-c-curve/output_file",
                    "id": "#main/output_preseq_c_curve_files"
                },
                {
                    "doc": "Read counts of the mapped BAM files",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/mapped_reads_count/output",
                    "id": "#main/output_read_count_mapped"
                },
                {
                    "doc": "Read counts of the mapped and filtered BAM files",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "outputSource": "#main/mapped_filtered_reads_count/output_read_count",
                    "id": "#main/output_read_count_mapped_filtered"
                }
            ]
        },
        {
            "class": "CommandLineTool",
            "doc": "Tool:    bedtools genomecov (aka genomeCoverageBed)\nVersion: v2.25.0\nSummary: Compute the coverage of a feature file among a genome.\n\nUsage: bedtools genomecov [OPTIONS] -i <bed/gff/vcf> -g <genome>\n\nOptions: \n\t-ibam\t\tThe input file is in BAM format.\n\t\t\tNote: BAM _must_ be sorted by position\n\n\t-d\t\tReport the depth at each genome position (with one-based coordinates).\n\t\t\tDefault behavior is to report a histogram.\n\n\t-dz\t\tReport the depth at each genome position (with zero-based coordinates).\n\t\t\tReports only non-zero positions.\n\t\t\tDefault behavior is to report a histogram.\n\n\t-bg\t\tReport depth in BedGraph format. For details, see:\n\t\t\tgenome.ucsc.edu/goldenPath/help/bedgraph.html\n\n\t-bga\t\tReport depth in BedGraph format, as above (-bg).\n\t\t\tHowever with this option, regions with zero \n\t\t\tcoverage are also reported. This allows one to\n\t\t\tquickly extract all regions of a genome with 0 \n\t\t\tcoverage by applying: \"grep -w 0$\" to the output.\n\n\t-split\t\tTreat \"split\" BAM or BED12 entries as distinct BED intervals.\n\t\t\twhen computing coverage.\n\t\t\tFor BAM files, this uses the CIGAR \"N\" and \"D\" operations \n\t\t\tto infer the blocks for computing coverage.\n\t\t\tFor BED12 files, this uses the BlockCount, BlockStarts, and BlockEnds\n\t\t\tfields (i.e., columns 10,11,12).\n\n\t-strand\t\tCalculate coverage of intervals from a specific strand.\n\t\t\tWith BED files, requires at least 6 columns (strand is column 6). \n\t\t\t- (STRING): can be + or -\n\n\t-5\t\tCalculate coverage of 5\" positions (instead of entire interval).\n\n\t-3\t\tCalculate coverage of 3\" positions (instead of entire interval).\n\n\t-max\t\tCombine all positions with a depth >= max into\n\t\t\ta single bin in the histogram. Irrelevant\n\t\t\tfor -d and -bedGraph\n\t\t\t- (INTEGER)\n\n\t-scale\t\tScale the coverage by a constant factor.\n\t\t\tEach coverage value is multiplied by this factor before being reported.\n\t\t\tUseful for normalizing coverage by, e.g., reads per million (RPM).\n\t\t\t- Default is 1.0; i.e., unscaled.\n\t\t\t- (FLOAT)\n\n\t-trackline\tAdds a UCSC/Genome-Browser track line definition in the first line of the output.\n\t\t\t- See here for more details about track line definition:\n\t\t\t      http://genome.ucsc.edu/goldenPath/help/bedgraph.html\n\t\t\t- NOTE: When adding a trackline definition, the output BedGraph can be easily\n\t\t\t      uploaded to the Genome Browser as a custom track,\n\t\t\t      BUT CAN NOT be converted into a BigWig file (w/o removing the first line).\n\n\t-trackopts\tWrites additional track line definition parameters in the first line.\n\t\t\t- Example:\n\t\t\t   -trackopts 'name=\"My Track\" visibility=2 color=255,30,30'\n\t\t\t   Note the use of single-quotes if you have spaces in your parameters.\n\t\t\t- (TEXT)\n\nNotes: \n\t(1) The genome file should tab delimited and structured as follows:\n\t <chromName><TAB><chromSize>\n\n\tFor example, Human (hg19):\n\tchr1\t249250621\n\tchr2\t243199373\n\t...\n\tchr18_gl000207_random\t4262\n\n\t(2) The input BED (-i) file must be grouped by chromosome.\n\t A simple \"sort -k 1,1 <BED> > <BED>.sorted\" will suffice.\n\n\t(3) The input BAM (-ibam) file must be sorted by position.\n\t A \"samtools sort <BAM>\" should suffice.\n\nTips: \n\tOne can use the UCSC Genome Browser's MySQL database to extract\n\tchromosome sizes. For example, H. sapiens:\n\n\tmysql --user=genome --host=genome-mysql.cse.ucsc.edu -A -e \\\n\t\"select chrom, size from hg19.chromInfo\" > hg19.genome",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ShellCommandRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "dukegcb/bedtools",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-3"
                    },
                    "doc": "\tCalculate coverage of 3\" positions (instead of entire interval).\n",
                    "id": "#bedtools-genomecov.cwl/3"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-5"
                    },
                    "doc": "\tCalculate coverage of 5\" positions (instead of entire interval).\n",
                    "id": "#bedtools-genomecov.cwl/5"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-bg"
                    },
                    "doc": "\tReport depth in BedGraph format. For details, see:\ngenome.ucsc.edu/goldenPath/help/bedgraph.html\n",
                    "id": "#bedtools-genomecov.cwl/bg"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-bga"
                    },
                    "doc": "\tReport depth in BedGraph format, as above (-bg).\nHowever with this option, regions with zero\ncoverage are also reported. This allows one to\nquickly extract all regions of a genome with 0\ncoverage by applying: \"grep -w 0$\" to the output.\n",
                    "id": "#bedtools-genomecov.cwl/bga"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-d"
                    },
                    "doc": "\tReport the depth at each genome position (with one-based coordinates).\nDefault behavior is to report a histogram.\n",
                    "id": "#bedtools-genomecov.cwl/d"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-dz"
                    },
                    "doc": "\tReport the depth at each genome position (with zero-based coordinates).\nReports only non-zero positions.\nDefault behavior is to report a histogram.\n",
                    "id": "#bedtools-genomecov.cwl/dz"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 3,
                        "prefix": "-g"
                    },
                    "doc": "<genome sizes>",
                    "id": "#bedtools-genomecov.cwl/g"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-ibam"
                    },
                    "doc": "\tThe input file is in BAM format.\nNote: BAM _must_ be sorted by position\n",
                    "id": "#bedtools-genomecov.cwl/ibam"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-max"
                    },
                    "doc": "\tCombine all positions with a depth >= max into\na single bin in the histogram. Irrelevant\nfor -d and -bedGraph\n- (INTEGER)\n",
                    "id": "#bedtools-genomecov.cwl/max"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-scale"
                    },
                    "doc": "\tScale the coverage by a constant factor.\nEach coverage value is multiplied by this factor before being reported.\nUseful for normalizing coverage by, e.g., reads per million (RPM).\n- Default is 1.0; i.e., unscaled.\n- (FLOAT)\n",
                    "id": "#bedtools-genomecov.cwl/scale"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-split"
                    },
                    "doc": "\tTreat \"split\" BAM or BED12 entries as distinct BED intervals.\nwhen computing coverage.\nFor BAM files, this uses the CIGAR \"N\" and \"D\" operations\nto infer the blocks for computing coverage.\nFor BED12 files, this uses the BlockCount, BlockStarts, and BlockEnds\nfields (i.e., columns 10,11,12).\n",
                    "id": "#bedtools-genomecov.cwl/split"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-strand"
                    },
                    "doc": "\tCalculate coverage of intervals from a specific strand.\nWith BED files, requires at least 6 columns (strand is column 6).\n- (STRING): can be + or -\n",
                    "id": "#bedtools-genomecov.cwl/strand"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-trackline"
                    },
                    "doc": "Adds a UCSC/Genome-Browser track line definition in the first line of the output.\n- See here for more details about track line definition:\nhttp://genome.ucsc.edu/goldenPath/help/bedgraph.html\n- NOTE: When adding a trackline definition, the output BedGraph can be easily\nuploaded to the Genome Browser as a custom track,\nBUT CAN NOT be converted into a BigWig file (w/o removing the first line).\n",
                    "id": "#bedtools-genomecov.cwl/trackline"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-trackopts"
                    },
                    "doc": "Writes additional track line definition parameters in the first line.\n- Example:\n-trackopts 'name=\"My Track\" visibility=2 color=255,30,30'\nNote the use of single-quotes if you have spaces in your parameters.\n- (TEXT)\n",
                    "id": "#bedtools-genomecov.cwl/trackopts"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.ibam.path.replace(/^.*[\\\\\\/]/, '').replace(/\\.[^/.]+$/, '') + '.bdg')"
                    },
                    "id": "#bedtools-genomecov.cwl/output_bedfile"
                }
            ],
            "baseCommand": [
                "bedtools",
                "genomecov"
            ],
            "stdout": "$(inputs.ibam.path.replace(/^.*[\\\\\\/]/, '').replace(/\\.[^/.]+$/, '') + '.bdg')",
            "id": "#bedtools-genomecov.cwl"
        },
        {
            "class": "CommandLineTool",
            "doc": "Get number of processed reads from Bowtie log.",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ShellCommandRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "reddylab/workflow-utils:ggr",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {},
                    "id": "#bowtie-log-read-count.cwl/bowtie_log"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.bowtie_log.path.replace(/^.*[\\\\\\/]/, '') + '.read_count.mapped')"
                    },
                    "id": "#bowtie-log-read-count.cwl/output"
                }
            ],
            "baseCommand": "read-count-from-bowtie-log.sh",
            "stdout": "$(inputs.bowtie_log.path.replace(/^.*[\\\\\\/]/, '') + '.read_count.mapped')",
            "id": "#bowtie-log-read-count.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ShellCommandRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "dukegcb/bowtie",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "default": 2000,
                    "inputBinding": {
                        "position": 4,
                        "prefix": "-X"
                    },
                    "doc": "maximum insert size for paired-end alignment (default: 2000)",
                    "id": "#bowtie-pe.cwl/X"
                },
                {
                    "type": "boolean",
                    "default": true,
                    "inputBinding": {
                        "position": 5,
                        "prefix": "--best"
                    },
                    "doc": "Hits guaranteed best stratum; ties broken by quality",
                    "id": "#bowtie-pe.cwl/best"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 5,
                        "prefix": "--chunkmbs"
                    },
                    "doc": "The number of megabytes of memory a given thread is given to store path descriptors in --best mode. (Default: 256)",
                    "id": "#bowtie-pe.cwl/chunkmbs"
                },
                {
                    "type": "File",
                    "secondaryFiles": [
                        "^^.2.ebwt",
                        "^^.3.ebwt",
                        "^^.4.ebwt",
                        "^^.rev.1.ebwt",
                        "^^.rev.2.ebwt"
                    ],
                    "inputBinding": {
                        "position": 9,
                        "valueFrom": "$(self.path.split('.').splice(0,self.path.split('.').length-2).join(\".\"))"
                    },
                    "doc": "First file (extension .1.ebwt) of the Bowtie index files generated for the reference genome (see http://bowtie-bio.sourceforge.net/tutorial.shtml#newi)",
                    "id": "#bowtie-pe.cwl/genome_ref_first_index_file"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 10,
                        "prefix": "-1"
                    },
                    "doc": "Query input FASTQ file.",
                    "id": "#bowtie-pe.cwl/input_fastq_read1_file"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 11,
                        "prefix": "-2"
                    },
                    "doc": "Query input FASTQ file.",
                    "id": "#bowtie-pe.cwl/input_fastq_read2_file"
                },
                {
                    "type": "int",
                    "default": 1,
                    "inputBinding": {
                        "position": 7,
                        "prefix": "-m"
                    },
                    "doc": "Suppress all alignments if > <int> exist (def: 1)",
                    "id": "#bowtie-pe.cwl/m"
                },
                {
                    "type": "int",
                    "default": 1,
                    "inputBinding": {
                        "position": 8,
                        "prefix": "--threads"
                    },
                    "doc": "<int> number of alignment threads to launch (default: 1)",
                    "id": "#bowtie-pe.cwl/nthreads"
                },
                {
                    "type": "string",
                    "id": "#bowtie-pe.cwl/output_filename"
                },
                {
                    "type": "boolean",
                    "default": true,
                    "inputBinding": {
                        "position": 2,
                        "prefix": "--sam"
                    },
                    "doc": "Write hits in SAM format (default: BAM)",
                    "id": "#bowtie-pe.cwl/sam"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedlen"
                    },
                    "doc": "seed length for -n (default: 28)",
                    "id": "#bowtie-pe.cwl/seedlen"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--seedmms"
                    },
                    "doc": "max mismatches in seed (between [0, 3], default: 2)",
                    "id": "#bowtie-pe.cwl/seedmms"
                },
                {
                    "type": "boolean",
                    "default": true,
                    "inputBinding": {
                        "position": 6,
                        "prefix": "--strata"
                    },
                    "doc": "Hits in sub-optimal strata aren't reported (requires --best)",
                    "id": "#bowtie-pe.cwl/strata"
                },
                {
                    "type": "boolean",
                    "default": true,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-t"
                    },
                    "doc": "Print wall-clock time taken by search phases",
                    "id": "#bowtie-pe.cwl/t"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--trim3"
                    },
                    "doc": "trim <int> bases from 3' (right) end of reads",
                    "id": "#bowtie-pe.cwl/trim3"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "--trim5"
                    },
                    "doc": "trim <int> bases from 5' (left) end of reads",
                    "id": "#bowtie-pe.cwl/trim5"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "default": 2,
                    "inputBinding": {
                        "position": 3,
                        "prefix": "-v"
                    },
                    "doc": "Report end-to-end hits w/ <=v mismatches; ignore qualities",
                    "id": "#bowtie-pe.cwl/v"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_filename + '.sam')"
                    },
                    "doc": "Aligned bowtie file in [SAM|BAM] format.",
                    "id": "#bowtie-pe.cwl/output_aligned_file"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_filename + '.bowtie.log')"
                    },
                    "id": "#bowtie-pe.cwl/output_bowtie_log"
                }
            ],
            "baseCommand": "bowtie",
            "stderr": "$(inputs.output_filename + '.bowtie.log')",
            "arguments": [
                {
                    "valueFrom": "$(inputs.output_filename + '.sam')",
                    "position": 12
                }
            ],
            "id": "#bowtie-pe.cwl"
        },
        {
            "class": "CommandLineTool",
            "doc": "Compute PCR Bottleneck Coeficient from BedGraph file.",
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1
                    },
                    "id": "#compute-pbc.cwl/bedgraph_file"
                },
                {
                    "type": "string",
                    "id": "#compute-pbc.cwl/output_filename"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_filename + '.PBC.txt')"
                    },
                    "id": "#compute-pbc.cwl/pbc"
                }
            ],
            "baseCommand": [
                "awk",
                "$4==1 {N1 += $3 - $2}; $4>=1 {Nd += $3 - $2} END {print N1/Nd}"
            ],
            "stdout": "$(inputs.output_filename + '.PBC.txt')",
            "id": "#compute-pbc.cwl"
        },
        {
            "class": "Workflow",
            "doc": "ChIP-seq - map - PCR Bottleneck Coefficients",
            "requirements": [
                {
                    "class": "ScatterFeatureRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "id": "#pcr-bottleneck-coef.cwl/genome_sizes"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "id": "#pcr-bottleneck-coef.cwl/input_bam_files"
                },
                {
                    "type": {
                        "type": "array",
                        "items": "string"
                    },
                    "id": "#pcr-bottleneck-coef.cwl/input_output_filenames"
                }
            ],
            "steps": [
                {
                    "run": "#bedtools-genomecov.cwl",
                    "in": [
                        {
                            "default": true,
                            "id": "#pcr-bottleneck-coef.cwl/bedtools_genomecov/bg"
                        },
                        {
                            "source": "#pcr-bottleneck-coef.cwl/genome_sizes",
                            "id": "#pcr-bottleneck-coef.cwl/bedtools_genomecov/g"
                        },
                        {
                            "source": "#pcr-bottleneck-coef.cwl/input_bam_files",
                            "id": "#pcr-bottleneck-coef.cwl/bedtools_genomecov/ibam"
                        }
                    ],
                    "scatter": "#pcr-bottleneck-coef.cwl/bedtools_genomecov/ibam",
                    "out": [
                        "#pcr-bottleneck-coef.cwl/bedtools_genomecov/output_bedfile"
                    ],
                    "id": "#pcr-bottleneck-coef.cwl/bedtools_genomecov"
                },
                {
                    "run": "#compute-pbc.cwl",
                    "in": [
                        {
                            "source": "#pcr-bottleneck-coef.cwl/bedtools_genomecov/output_bedfile",
                            "id": "#pcr-bottleneck-coef.cwl/compute_pbc/bedgraph_file"
                        },
                        {
                            "source": "#pcr-bottleneck-coef.cwl/input_output_filenames",
                            "id": "#pcr-bottleneck-coef.cwl/compute_pbc/output_filename"
                        }
                    ],
                    "scatterMethod": "dotproduct",
                    "scatter": [
                        "#pcr-bottleneck-coef.cwl/compute_pbc/bedgraph_file",
                        "#pcr-bottleneck-coef.cwl/compute_pbc/output_filename"
                    ],
                    "out": [
                        "#pcr-bottleneck-coef.cwl/compute_pbc/pbc"
                    ],
                    "id": "#pcr-bottleneck-coef.cwl/compute_pbc"
                }
            ],
            "outputs": [
                {
                    "outputSource": "#pcr-bottleneck-coef.cwl/compute_pbc/pbc",
                    "type": {
                        "type": "array",
                        "items": "File"
                    },
                    "id": "#pcr-bottleneck-coef.cwl/pbc_file"
                }
            ],
            "id": "#pcr-bottleneck-coef.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ShellCommandRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "dukegcb/picard",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 4,
                        "valueFrom": "$('INPUT=' + self.path)",
                        "shellQuote": false
                    },
                    "doc": "One or more input SAM or BAM files to analyze. Must be coordinate sorted.",
                    "id": "#picard-MarkDuplicates.cwl/input_file"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "shellQuote": false
                    },
                    "doc": "JVM arguments should be a quoted, space separated list (e.g. \"-Xms128m -Xmx512m\")",
                    "id": "#picard-MarkDuplicates.cwl/java_opts"
                },
                {
                    "type": "string",
                    "default": "dedup_metrics.txt",
                    "doc": "Suffix used to create the metrics output file (Default: dedup_metrics.txt)",
                    "id": "#picard-MarkDuplicates.cwl/metrics_suffix"
                },
                {
                    "type": "string",
                    "doc": "Output filename used as basename",
                    "id": "#picard-MarkDuplicates.cwl/output_filename"
                },
                {
                    "type": "string",
                    "default": "dedup.bam",
                    "doc": "Suffix used to identify the output file (Default: dedup.bam)",
                    "id": "#picard-MarkDuplicates.cwl/output_suffix"
                },
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 2,
                        "prefix": "-jar"
                    },
                    "doc": "Path to the picard.jar file",
                    "id": "#picard-MarkDuplicates.cwl/picard_jar_path"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "valueFrom": "$('REMOVE_DUPLICATES=' + self)",
                        "position": 5
                    },
                    "doc": "If true do not write duplicates to the output file instead of writing them with appropriate flags set.  (Default false).",
                    "id": "#picard-MarkDuplicates.cwl/remove_duplicates"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_filename  + '.' + inputs.output_suffix)"
                    },
                    "id": "#picard-MarkDuplicates.cwl/output_dedup_bam_file"
                },
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_filename + '.' + inputs.metrics_suffix)"
                    },
                    "id": "#picard-MarkDuplicates.cwl/output_metrics_file"
                }
            ],
            "baseCommand": [
                "java"
            ],
            "arguments": [
                {
                    "valueFrom": "MarkDuplicates",
                    "position": 3
                },
                {
                    "valueFrom": "$('OUTPUT=' + inputs.output_filename + '.' + inputs.output_suffix)",
                    "position": 5,
                    "shellQuote": false
                },
                {
                    "valueFrom": "$('METRICS_FILE='+inputs.output_filename + '.' + inputs.metrics_suffix)",
                    "position": 5,
                    "shellQuote": false
                },
                {
                    "valueFrom": "$('TMP_DIR='+runtime.tmpdir)",
                    "position": 5,
                    "shellQuote": false
                }
            ],
            "id": "#picard-MarkDuplicates.cwl"
        },
        {
            "class": "CommandLineTool",
            "doc": "Usage: c_curve [OPTIONS] <sorted-bed-file>\n\nOptions:\n  -o, -output   yield output file (default: stdout) \n  -s, -step     step size in extrapolations (default: 1e+06) \n  -v, -verbose  print more information \n  -P, -pe       input is paired end read file \n  -H, -hist     input is a text file containing the observed histogram \n  -V, -vals     input is a text file containing only the observed counts \n  -B, -bam      input is in BAM format \n  -l, -seg_len  maximum segment length when merging paired end bam reads \n                (default: 5000) \n\nHelp options:\n  -?, -help     print this help message \n      -about    print about message",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "reddylab/preseq:2.0",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "boolean",
                    "default": true,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-B"
                    },
                    "doc": "-bam      input is in BAM format \n",
                    "id": "#preseq-c_curve.cwl/B"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-H"
                    },
                    "doc": "-hist     input is a text file containing the observed histogram \n",
                    "id": "#preseq-c_curve.cwl/H"
                },
                {
                    "type": [
                        "null",
                        "File"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-V"
                    },
                    "doc": "-vals     input is a text file containing only the observed counts \n",
                    "id": "#preseq-c_curve.cwl/V"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 2
                    },
                    "doc": "Sorted bed or BAM file",
                    "id": "#preseq-c_curve.cwl/input_sorted_file"
                },
                {
                    "type": [
                        "null",
                        "int"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-l"
                    },
                    "doc": "-seg_len  maximum segment length when merging paired end bam reads \n(default: 5000)\nHelp options:\n-?, -help     print this help message\n-about    print about message\n",
                    "id": "#preseq-c_curve.cwl/l"
                },
                {
                    "type": "string",
                    "id": "#preseq-c_curve.cwl/output_file_basename"
                },
                {
                    "type": [
                        "null",
                        "boolean"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-P"
                    },
                    "doc": "-pe       input is paired end read file \n",
                    "id": "#preseq-c_curve.cwl/pe"
                },
                {
                    "type": [
                        "null",
                        "float"
                    ],
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-s"
                    },
                    "doc": "-step     step size in extrapolations (default: 1e+06) \n",
                    "id": "#preseq-c_curve.cwl/s"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-v"
                    },
                    "doc": "-verbose  print more information \n",
                    "id": "#preseq-c_curve.cwl/v"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_file_basename + '.preseq_c_curve.txt')"
                    },
                    "id": "#preseq-c_curve.cwl/output_file"
                }
            ],
            "baseCommand": [
                "preseq",
                "c_curve"
            ],
            "stdout": "$(inputs.output_file_basename + '.preseq_c_curve.txt')",
            "id": "#preseq-c_curve.cwl"
        },
        {
            "class": "CommandLineTool",
            "doc": "Get number of processed reads from Bowtie log.",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ShellCommandRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "reddylab/workflow-utils:ggr",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {},
                    "id": "#preseq-percent-uniq-reads.cwl/preseq_c_curve_outfile"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.preseq_c_curve_outfile.path.replace(/^.*[\\\\\\/]/, '').replace(/\\.[^/.]+$/, \"\") + '.percentage_unique_reads.txt')"
                    },
                    "id": "#preseq-percent-uniq-reads.cwl/output"
                }
            ],
            "baseCommand": "percent-uniq-reads-from-preseq.sh",
            "stdout": "$(inputs.preseq_c_curve_outfile.path.replace(/^.*[\\\\\\/]/, '').replace(/\\.[^/.]+$/, \"\") + '.percentage_unique_reads.txt')",
            "id": "#preseq-percent-uniq-reads.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "dukegcb/samtools",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1000
                    },
                    "doc": "Aligned file to be sorted with samtools",
                    "id": "#samtools-filter-unmapped.cwl/input_file"
                },
                {
                    "type": "int",
                    "default": 1,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-@"
                    },
                    "doc": "Number of threads used in sorting",
                    "id": "#samtools-filter-unmapped.cwl/nthreads"
                },
                {
                    "type": "string",
                    "doc": "Basename for the output file",
                    "id": "#samtools-filter-unmapped.cwl/output_filename"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.output_filename + '.accepted_hits.bam')"
                    },
                    "doc": "Filter unmapped reads in aligned file",
                    "id": "#samtools-filter-unmapped.cwl/filtered_file"
                }
            ],
            "baseCommand": [
                "samtools",
                "view",
                "-F",
                "4",
                "-b",
                "-h"
            ],
            "stdout": "$(inputs.output_filename + '.accepted_hits.bam')",
            "id": "#samtools-filter-unmapped.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "listing": [
                        "$(inputs.bam)"
                    ],
                    "class": "InitialWorkDirRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "dukegcb/samtools:1.3",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1
                    },
                    "secondaryFiles": [
                        ".bai"
                    ],
                    "doc": "Bam file (it should be indexed)",
                    "id": "#samtools-idxstats.cwl/bam"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "doc": "Idxstats output file. TAB-delimited with each line consisting of reference\nsequence name, sequence length, # mapped reads and # unmapped reads\n",
                    "outputBinding": {
                        "glob": "$(inputs.bam.basename + \".idxstats\")"
                    },
                    "id": "#samtools-idxstats.cwl/idxstats_file"
                }
            ],
            "baseCommand": [
                "samtools",
                "idxstats"
            ],
            "stdout": "$(inputs.bam.basename + \".idxstats\")",
            "id": "#samtools-idxstats.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "listing": [
                        "$(inputs.input_file)"
                    ],
                    "class": "InitialWorkDirRequirement"
                },
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "dukegcb/samtools",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1
                    },
                    "doc": "Aligned file to be sorted with samtools",
                    "id": "#samtools-index.cwl/input_file"
                }
            ],
            "outputs": [
                {
                    "doc": "Indexed BAM file",
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.input_file.basename)"
                    },
                    "secondaryFiles": ".bai",
                    "id": "#samtools-index.cwl/indexed_file"
                }
            ],
            "baseCommand": [
                "samtools",
                "index"
            ],
            "arguments": [
                {
                    "valueFrom": "$(inputs.input_file.basename + '.bai')",
                    "position": 2
                }
            ],
            "id": "#samtools-index.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "dukegcb/samtools",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1000
                    },
                    "doc": "Aligned file to be sorted with samtools",
                    "id": "#samtools-sort.cwl/input_file"
                },
                {
                    "type": "boolean",
                    "default": false,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-n"
                    },
                    "doc": "Sort by read name",
                    "id": "#samtools-sort.cwl/n"
                },
                {
                    "type": "int",
                    "default": 1,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-@"
                    },
                    "doc": "Number of threads used in sorting",
                    "id": "#samtools-sort.cwl/nthreads"
                },
                {
                    "type": "string",
                    "default": ".sorted.bam",
                    "doc": "suffix of the transformed SAM/BAM file (including extension, e.g. .filtered.sam)",
                    "id": "#samtools-sort.cwl/suffix"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.input_file.path.replace(/^.*[\\\\\\/]/, '').replace(/\\.[^/.]+$/, '') + inputs.suffix)"
                    },
                    "doc": "Sorted aligned file",
                    "id": "#samtools-sort.cwl/sorted_file"
                }
            ],
            "baseCommand": [
                "samtools",
                "sort"
            ],
            "stdout": "$(inputs.input_file.path.replace(/^.*[\\\\\\/]/, '').replace(/\\.[^/.]+$/, '') + inputs.suffix)",
            "id": "#samtools-sort.cwl"
        },
        {
            "class": "CommandLineTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "dukegcb/samtools",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "boolean",
                    "default": true,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-S"
                    },
                    "doc": "Input format autodetected",
                    "id": "#samtools2bam.cwl/S"
                },
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 2
                    },
                    "doc": "File to be converted to BAM with samtools",
                    "id": "#samtools2bam.cwl/input_file"
                },
                {
                    "type": "int",
                    "default": 1,
                    "inputBinding": {
                        "position": 1,
                        "prefix": "-@"
                    },
                    "doc": "Number of threads used",
                    "id": "#samtools2bam.cwl/nthreads"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.input_file.path.replace(/^.*[\\\\\\/]/, '').replace(/\\.[^/.]+$/, '') + '.bam')"
                    },
                    "doc": "Aligned file in BAM format",
                    "id": "#samtools2bam.cwl/bam_file"
                }
            ],
            "baseCommand": [
                "samtools",
                "view",
                "-b"
            ],
            "stdout": "$(inputs.input_file.path.replace(/^.*[\\\\\\/]/, '').replace(/\\.[^/.]+$/, '') + '.bam')",
            "id": "#samtools2bam.cwl"
        },
        {
            "class": "CommandLineTool",
            "doc": "Extract mapped reads from BAM file using Samtools flagstat command",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                },
                {
                    "class": "ShellCommandRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "dukegcb/samtools",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1
                    },
                    "doc": "Aligned BAM file to filter",
                    "id": "#samtools-extract-number-mapped-reads.cwl/input_bam_file"
                },
                {
                    "type": "string",
                    "id": "#samtools-extract-number-mapped-reads.cwl/output_suffix"
                }
            ],
            "outputs": [
                {
                    "type": "File",
                    "outputBinding": {
                        "glob": "$(inputs.input_bam_file.path.replace(/^.*[\\\\\\/]/, '').replace(/\\.[^/.]+$/, \"\") + inputs.output_suffix)"
                    },
                    "doc": "Samtools Flagstat report file",
                    "id": "#samtools-extract-number-mapped-reads.cwl/output_read_count"
                }
            ],
            "baseCommand": [
                "samtools",
                "flagstat"
            ],
            "arguments": [
                {
                    "valueFrom": " | head -n1 | cut -f 1 -d ' '",
                    "position": 10000,
                    "shellQuote": false
                }
            ],
            "stdout": "$(inputs.input_bam_file.path.replace(/^.*[\\\\\\/]/, '').replace(/\\.[^/.]+$/, \"\") + inputs.output_suffix)",
            "id": "#samtools-extract-number-mapped-reads.cwl"
        },
        {
            "class": "CommandLineTool",
            "doc": "Extracts the base name of a file",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "hints": [
                {
                    "dockerPull": "reddylab/workflow-utils:ggr",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "File",
                    "inputBinding": {
                        "position": 1
                    },
                    "id": "#extract-basename.cwl/input_file"
                }
            ],
            "outputs": [
                {
                    "type": "string",
                    "outputBinding": {
                        "outputEval": "$(inputs.input_file.path.substr(inputs.input_file.path.lastIndexOf('/') + 1, inputs.input_file.path.lastIndexOf('.') - (inputs.input_file.path.lastIndexOf('/') + 1)))"
                    },
                    "id": "#extract-basename.cwl/output_basename"
                }
            ],
            "baseCommand": "echo",
            "id": "#extract-basename.cwl"
        },
        {
            "class": "ExpressionTool",
            "requirements": [
                {
                    "class": "InlineJavascriptRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "string",
                    "doc": "Query chromosome used to calculate percentage",
                    "id": "#idxstats-percentage-of-reads-in-chrom.cwl/chrom"
                },
                {
                    "type": "File",
                    "doc": "Samtools idxstats file",
                    "inputBinding": {
                        "loadContents": true
                    },
                    "id": "#idxstats-percentage-of-reads-in-chrom.cwl/idxstats"
                },
                {
                    "type": [
                        "null",
                        "string"
                    ],
                    "doc": "Save the percentage in a file of the given name",
                    "id": "#idxstats-percentage-of-reads-in-chrom.cwl/output_filename"
                }
            ],
            "expression": "${\n  var regExp = new RegExp(inputs.chrom + \"\\\\s\\\\d+\\\\s(\\\\d+)\\\\s(\\\\d+)\");\n  var match = inputs.idxstats.contents.match(regExp);\n  if (match){\n    var chrom_mapped_reads = match[1];\n    var total_reads = inputs.idxstats.contents.split(\"\\r\")\n      .map(function(x){\n        var rr = x.match(/.*\\s\\d+\\s(\\d+)\\s\\d+/);\n        return (rr ? rr[1] : 0);\n      })\n      .reduce(function(a, b) { return Number(a) + Number(b); });\n\n    var output = (100*chrom_mapped_reads/total_reads).toFixed(4) + \"%\";\n\n    if (inputs.output_filename){\n      return {\n        percent_map : {\n          \"class\": \"File\",\n          \"basename\" : inputs.output_filename,\n          \"contents\" : output,\n        }\n      }\n    }\n    return output;\n  }\n}\n",
            "outputs": [
                {
                    "type": [
                        "File",
                        "string"
                    ],
                    "id": "#idxstats-percentage-of-reads-in-chrom.cwl/percent_map"
                }
            ],
            "id": "#idxstats-percentage-of-reads-in-chrom.cwl"
        },
        {
            "class": "CommandLineTool",
            "doc": "Extracts the base name of a file",
            "hints": [
                {
                    "dockerPull": "reddylab/workflow-utils:ggr",
                    "class": "DockerRequirement"
                }
            ],
            "inputs": [
                {
                    "type": "string",
                    "inputBinding": {
                        "position": 1
                    },
                    "id": "#remove-extension.cwl/file_path"
                }
            ],
            "outputs": [
                {
                    "type": "string",
                    "outputBinding": {
                        "outputEval": "$(inputs.file_path.replace(/\\.[^/.]+$/, \"\"))"
                    },
                    "id": "#remove-extension.cwl/output_path"
                }
            ],
            "baseCommand": "echo",
            "id": "#remove-extension.cwl"
        }
    ],
    "cwlVersion": "v1.0"
}
